@@ -51,7 +51,7 @@ class CdoTest(unittest.TestCase):
 
     def testVersions(self):
         cdo = Cdo()
-        self.assertEqual('1.3.5',cdo.__version__())
+        self.assertEqual('1.3.6',cdo.__version__())
         self.assertTrue(parse_version('1.7.0') <= parse_version(cdo.version()))
 
     def testCDO(self):
@@ -74,6 +74,21 @@ class CdoTest(unittest.TestCase):
         cdo = Cdo(cdfMod=CDF_MOD)
         print(cdo.version(verbose=True))
 
+    def test_hasCdo(self):
+        cdo = Cdo()
+        self.assertTrue(cdo.hasCdo())
+        cdo.CDO='cccccccc'
+        self.assertFalse(cdo.hasCdo())
+        cdo.CDO='/bin/cdo'
+        if os.path.isfile(cdo.CDO):
+          self.assertTrue(cdo.hasCdo())
+
+    def test_check(self):
+        cdo = Cdo()
+        self.assertTrue(cdo.check())
+        cdo.CDO='cvcvcvcvc'
+        self.assertFalse(cdo.check())
+
     def testOps(self):
         cdo = Cdo(cdfMod=CDF_MOD)
         self.assertTrue("sinfov" in cdo.operators)
@@ -300,8 +315,8 @@ class CdoTest(unittest.TestCase):
         cdo = Cdo(cdfMod=CDF_MOD)
         cdo.debug = DEBUG
         self.assertEqual(False, cdo.stdatm(0,options = '-f nc', returnArray = 'TT'))
-#TODO       temperature = cdo.stdatm(0,options = '-f nc', returnArray = 'T')
-#TODO       self.assertEqual(288.0,temperature.flatten()[0])
+        temperature = cdo.stdatm(0,options = '-f nc', returnArray = 'T')
+        self.assertEqual(288.0,temperature.flatten()[0])
 #TODO       pressure = cdo.stdatm("0,1000",options = '-f nc -b F64',returnArray = 'P')
 #TODO       self.assertEqual("[ 1013.25         898.54345604]",pressure.flatten().__str__())
 
@@ -402,7 +417,8 @@ class CdoTest(unittest.TestCase):
         cdo.__print__('test_splitOps')
         resultsFiles = cdo.splitname(input = '-stdatm,0',output = pattern)
         self.assertTrue(2 <= len(resultsFiles))
-        print(resultsFiles)
+        if DEBUG:
+          print(resultsFiles)
         for var in ['T','P']:
           print(pattern+var+'.grb')
           self.assertTrue(pattern+var+'.grb' in resultsFiles)
@@ -410,7 +426,8 @@ class CdoTest(unittest.TestCase):
 
         pattern = 'sel'
         resultsFiles = cdo.splitsel(1,input = '-for,0,9',output = pattern)
-        print(resultsFiles)
+        if DEBUG:
+          print(resultsFiles)
         self.assertTrue(10 <= len(resultsFiles))
         rm(resultsFiles)
         for var in range(0,10):
@@ -420,7 +437,8 @@ class CdoTest(unittest.TestCase):
         pattern = 'lev'
         resultsFiles = cdo.splitlevel(input = '-stdatm,100,2000,5000',output = pattern)
         self.assertTrue(3 <= len(resultsFiles))
-        print(resultsFiles)
+        if DEBUG:
+          print(resultsFiles)
         rm(resultsFiles)
         for var in ['0100','2000','5000']:
           self.assertTrue(pattern+'00'+str(var)+'.grb' in resultsFiles)
@@ -434,9 +452,9 @@ class CdoTest(unittest.TestCase):
     def test_libs(self):
         cdo = Cdo(cdfMod=CDF_MOD)
         cdo.debug = DEBUG
-        print(cdo.libs)
+        if DEBUG:
+          print(cdo.libs)
         self.assertTrue(cdo.hasLib("cdi"),"CDI support missing")
-        self.assertTrue(cdo.hasLib("extra"),"netcdf4 support missing")
         self.assertTrue(cdo.hasLib("nc4"),"netcdf4 support missing")
         self.assertTrue(cdo.hasLib("netcdf"),"netcdf support missing")
         self.assertTrue(cdo.hasLib("udunits2"),"netcdf support missing")
@@ -451,13 +469,15 @@ class CdoTest(unittest.TestCase):
         self.assertTrue(cdo.returnNoneOnError,"'returnNoneOnError' is _not_ True after manual setting")
         ret  = cdo.sinfo(input="-topf")
         self.assertEqual(None,ret)
-        print(ret)
+        if DEBUG:
+          print(ret)
 
         cdo_ = Cdo(cdfMod=CDF_MOD, returnNoneOnError=True)
         self.assertTrue(cdo_.returnNoneOnError)
         ret  = cdo_.sinfo(input=" ifile.grb")
         self.assertEqual(None,ret)
-        print(ret)
+        if DEBUG:
+          print(ret)
 
     def test_initOptions(self):
         cdo = Cdo(cdfMod=CDF_MOD, debug=True)
@@ -481,6 +501,7 @@ class CdoTest(unittest.TestCase):
         # setup
         cdo = Cdo(cdfMod=CDF_MOD)
         cdo.debug = DEBUG
+        self.assertEqual(os.environ,cdo.env)
 
         cdo.__print__('test_env')
         # cdf default
@@ -515,7 +536,8 @@ class CdoTest(unittest.TestCase):
     def test_showMaArray(self):
         cdo = Cdo(cdfMod=CDF_MOD)
         cdo.debug = DEBUG
-        print(cdo)
+        if DEBUG:
+          print(cdo)
         bathy = cdo.setrtomiss(0,10000,
             input = cdo.topo(options='-f nc'),returnMaArray='topo')
         plot(bathy)
@@ -618,37 +640,33 @@ class CdoTest(unittest.TestCase):
 
     def test_log(self):
         cmd = '-fldmean -mul -random,r20x20 -topo,r20x20'
-        print('# logging with a real file')
+        if DEBUG:
+          print('# logging with a real file')
         cdo = Cdo(cdfMod=CDF_MOD,logging = True,logFile='foo.log')
         cdo.topo()
         cdo.temp()
         cdo.sinfov(input=cmd)
-        cdo.showLog()
+        if DEBUG:
+          cdo.showLog()
 
         cmd = '-fldmean -mul -random,r20x20 -topo,r20x20'
-        print('# logging with a real file, passed as unicode string')
+        if DEBUG:
+          print('# logging with a real file, passed as unicode string')
         cdo = Cdo(cdfMod=CDF_MOD, logging=True, logFile=u'foo.log')
         cdo.topo()
         cdo.temp()
         cdo.sinfov(input=cmd)
-        cdo.showLog()
+        if DEBUG:
+          cdo.showLog()
 
-        print('# logging with in-memory stringio')
+        if DEBUG:
+          print('# logging with in-memory stringio')
         cdo = Cdo(cdfMod=CDF_MOD,logging = True)
         cdo.topo()
         cdo.temp()
         cdo.sinfov(input=cmd)
-        cdo.showLog()
-
-    def test_zzzz(self):
-      cdo = Cdo(cdfMod=CDF_MOD)
-      cdo.__print__('test_zzzz')
-      self.assertEqual({},cdo.env)
-
-    def test_AAAA(self):
-      cdo = Cdo(cdfMod=CDF_MOD)
-      cdo.__print__('test_AAAA')
-      self.assertEqual({},cdo.env)
+        if DEBUG:
+          cdo.showLog()
 
     def test_noOutputOps(self):
       cdo = Cdo(cdfMod=CDF_MOD)
@@ -659,148 +677,163 @@ class CdoTest(unittest.TestCase):
     def test_cdiMeta(self):
       cdo = Cdo()
       ofile = cdo.stdatm("0", options = "-f nc", returnCdf = True)
-      print(ofile)
+      if DEBUG:
+        print(ofile)
       ofile = cdo.stdatm("0", options = "-f nc4", returnCdf = True)
-      print(ofile)
+      if DEBUG:
+        print(ofile)
       ofile = cdo.stdatm("0", options = "-f nc", returnXArray = 'T')
-      print(ofile)
-      print(ofile.attrs)
+      if DEBUG:
+        print(ofile)
+        print(ofile.attrs)
       ofile = cdo.stdatm("0", options = "-f nc", returnXDataset=True)
-      print(ofile)
-      print(ofile.attrs)
+      if DEBUG:
+        print(ofile)
+        print(ofile.attrs)
 
     if MAINTAINERMODE:
-        def test_icon_coords(self):
-          cdo = Cdo(cdfMod=CDF_MOD)
-          ifile = DATA_DIR +'/icon/oce_AquaAtlanticBoxACC.nc'
-          ivar  = 't_acc'
-          varIn = cdo.readCdf(ifile)
-          varIn = varIn.variables[ivar]
-          if ('scipy' == cdo.cdfMod ):
-            expected = b'clon clat'
-          else:
-            expected =  u'clon clat'
-          self.assertEqual(expected,varIn.coordinates)
 
-          varOut =cdo.readCdf(cdo.selname(ivar,input=ifile))
-          varOut = varOut.variables[ivar]
-          expected =  u'clat clon'
-          self.assertEqual(expected,varOut.coordinates)
-        def testCall(self):
-          cdo = Cdo(cdfMod=CDF_MOD)
+      def test_longChain(self):
+        cdo = Cdo(cdfMod=CDF_MOD)
+        ifile = "-enlarge,global_0.3 -settaxis,2000-01-01 -expr,'t=sin(for*3.141529/180.0)' -for,1,10"
+        t = cdo.fldmax(input="-div -sub -timmean -seltimestep,2,3 %s -seltimestep,1 %s -gridarea %s"%(ifile,ifile,ifile),returnArray="T")
+        self.assertEqual(8.9813e-09,t[0])
+
+      def test_icon_coords(self):
+        cdo = Cdo(cdfMod=CDF_MOD)
+        ifile = DATA_DIR +'/icon/oce_AquaAtlanticBoxACC.nc'
+        ivar  = 't_acc'
+        varIn = cdo.readCdf(ifile)
+        varIn = varIn.variables[ivar]
+        if ('scipy' == cdo.cdfMod ):
+          expected = b'clon clat'
+        else:
+          expected =  u'clon clat'
+        self.assertEqual(expected,varIn.coordinates)
+
+        varOut =cdo.readCdf(cdo.selname(ivar,input=ifile))
+        varOut = varOut.variables[ivar]
+        expected =  u'clat clon'
+        self.assertEqual(expected,varOut.coordinates)
+      def testCall(self):
+        cdo = Cdo(cdfMod=CDF_MOD)
+        if DEBUG:
           print(cdo.sinfov(input=DATA_DIR+'/icon/oce.nc'))
-        def test_readCdf(self):
-          cdo = Cdo(cdfMod=CDF_MOD)
-          input= "-settunits,days  -setyear,2000 -for,1,4"
-          cdfFile = cdo.copy(options="-f nc",input=input)
-          cdf     = cdo.readCdf(cdfFile)
+      def test_readCdf(self):
+        cdo = Cdo(cdfMod=CDF_MOD)
+        input= "-settunits,days  -setyear,2000 -for,1,4"
+        cdfFile = cdo.copy(options="-f nc",input=input)
+        cdf     = cdo.readCdf(cdfFile)
 
-          self.assertEqual(sorted(['lat','lon','for','time']),sorted(list(cdf.variables.keys())))
+        self.assertEqual(sorted(['lat','lon','for','time']),sorted(list(cdf.variables.keys())))
 
 
-        def test_phc(self):
+      def test_phc(self):
+        ifile = DATA_DIR+'/icon/phc.nc'
+        cdo = Cdo(cdfMod=CDF_MOD)
+        cdo.debug = DEBUG
+        #cdo.merge(input='/home/ram/data/icon/input/phc3.0/PHC__3.0__TempO__1x1__annual.nc /home/ram/data/icon/input/phc3.0/PHC__3.0__SO__1x1__annual.nc',
+        #          output=ifile,
+        #          options='-O')
+        s = cdo.sellonlatbox(0,30,0,90, input="-chname,SO,s,TempO,t " + ifile,output='test_my_phc.nc',returnMaArray='s',options='-f nc')
+        plot(np.flipud(s[0,:,:]),ofile='org',title='original')
+        sfmo = cdo.sellonlatbox(0,30,0,90, input="-fillmiss -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
+        plot(np.flipud(sfmo[0,:,:]),ofile='fm',title='fillmiss')
+        sfm = cdo.sellonlatbox(0,30,0,90, input="-fillmiss2 -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
+        plot(np.flipud(sfm[0,:,:]),ofile='fm2',title='fillmiss2')
+        ssetmisstonn = cdo.sellonlatbox(0,30,0,90, input="-setmisstonn -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
+        plot(np.flipud(ssetmisstonn[0,:,:]),ofile='setmisstonn',title='setmisstonn')
+        if (parse_version(cdo.version()) >= parse_version('1.7.2')):
+          smooth = cdo.sellonlatbox(0,30,0,90, input="-smooth -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
+          plot(np.flipud(ssetmisstonn[0,:,:]),ofile='smooth',title='smooth')
+        #global plot
+        #s_global = cdo.chname('SO,s,TempO,t',input=ifile,output='my_phc.nc',returnMaArray='s',options='-f nc')
+        #plot(s_global[0,:,:],ofile='org_global',title='org_global')
+        #sfmo_global = cdo.fillmiss(input=" -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
+        #plot(sfmo_global[0,:,:],ofile='fm_global',title='fm_global')
+        #sfm_global = cdo.fillmiss2(input=" -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
+        #plot(sfm_global[0,:,:],ofile='fm2_global',title='fm2_global')
+        #ssetmisstonn_global = cdo.setmisstonn(input=" -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
+        #plot(ssetmisstonn_global[0,:,:],ofile='setmisstonn_global',title='setmisstonn_global')
+
+      def test_smooth(self):
+        cdo = Cdo(cdfMod='netcdf4')
+        if (parse_version(cdo.version()) >= parse_version('1.7.2')):
           ifile = DATA_DIR+'/icon/phc.nc'
           cdo = Cdo(cdfMod=CDF_MOD)
           cdo.debug = DEBUG
           #cdo.merge(input='/home/ram/data/icon/input/phc3.0/PHC__3.0__TempO__1x1__annual.nc /home/ram/data/icon/input/phc3.0/PHC__3.0__SO__1x1__annual.nc',
           #          output=ifile,
           #          options='-O')
-          s = cdo.sellonlatbox(0,30,0,90, input="-chname,SO,s,TempO,t " + ifile,output='test_my_phc.nc',returnMaArray='s',options='-f nc')
-          plot(np.flipud(s[0,:,:]),ofile='org',title='original')
-          sfmo = cdo.sellonlatbox(0,30,0,90, input="-fillmiss -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
-          plot(np.flipud(sfmo[0,:,:]),ofile='fm',title='fillmiss')
-          sfm = cdo.sellonlatbox(0,30,0,90, input="-fillmiss2 -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
-          plot(np.flipud(sfm[0,:,:]),ofile='fm2',title='fillmiss2')
-          ssetmisstonn = cdo.sellonlatbox(0,30,0,90, input="-setmisstonn -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
-          plot(np.flipud(ssetmisstonn[0,:,:]),ofile='setmisstonn',title='setmisstonn')
-          if (parse_version(cdo.version()) >= parse_version('1.7.2')):
-            smooth = cdo.sellonlatbox(0,30,0,90, input="-smooth -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
-            plot(np.flipud(ssetmisstonn[0,:,:]),ofile='smooth',title='smooth')
-          #global plot
-          #s_global = cdo.chname('SO,s,TempO,t',input=ifile,output='my_phc.nc',returnMaArray='s',options='-f nc')
-          #plot(s_global[0,:,:],ofile='org_global',title='org_global')
-          #sfmo_global = cdo.fillmiss(input=" -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
-          #plot(sfmo_global[0,:,:],ofile='fm_global',title='fm_global')
-          #sfm_global = cdo.fillmiss2(input=" -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
-          #plot(sfm_global[0,:,:],ofile='fm2_global',title='fm2_global')
-          #ssetmisstonn_global = cdo.setmisstonn(input=" -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
-          #plot(ssetmisstonn_global[0,:,:],ofile='setmisstonn_global',title='setmisstonn_global')
+          smooth = cdo.smooth(input=" -sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
+          plot(np.flipud(smooth[0,:,:]),ofile='smooth',title='smooth')
 
-        def test_smooth(self):
-          cdo = Cdo(cdfMod='netcdf4')
-          if (parse_version(cdo.version()) >= parse_version('1.7.2')):
-            ifile = DATA_DIR+'/icon/phc.nc'
-            cdo = Cdo(cdfMod=CDF_MOD)
-            cdo.debug = DEBUG
-            #cdo.merge(input='/home/ram/data/icon/input/phc3.0/PHC__3.0__TempO__1x1__annual.nc /home/ram/data/icon/input/phc3.0/PHC__3.0__SO__1x1__annual.nc',
-            #          output=ifile,
-            #          options='-O')
-            smooth = cdo.smooth(input=" -sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
-            plot(np.flipud(smooth[0,:,:]),ofile='smooth',title='smooth')
+          smooth2 = cdo.smooth('nsmooth=2',input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
+          plot(np.flipud(smooth2[0,:,:]),ofile='smooth2',title='smooth,nsmooth=2')
 
-            smooth2 = cdo.smooth('nsmooth=2',input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
-            plot(np.flipud(smooth2[0,:,:]),ofile='smooth2',title='smooth,nsmooth=2')
+          smooth4 = cdo.smooth('nsmooth=4',input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
+          plot(np.flipud(smooth4[0,:,:]),ofile='smooth4',title='smooth,nsmooth=4')
 
-            smooth4 = cdo.smooth('nsmooth=4',input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
-            plot(np.flipud(smooth4[0,:,:]),ofile='smooth4',title='smooth,nsmooth=4')
+          smooth9 = cdo.smooth9(input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
+          plot(np.flipud(smooth9[0,:,:]),ofile='smooth9',title='smooth9')
 
-            smooth9 = cdo.smooth9(input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
-            plot(np.flipud(smooth9[0,:,:]),ofile='smooth9',title='smooth9')
+          smooth3deg = cdo.smooth('radius=6deg',input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
+          plot(np.flipud(smooth3deg[0,:,:]),ofile='smooth3deg',title='smooth,radius=6deg')
 
-            smooth3deg = cdo.smooth('radius=6deg',input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
-            plot(np.flipud(smooth3deg[0,:,:]),ofile='smooth3deg',title='smooth,radius=6deg')
+          smooth20 = cdo.smooth('nsmooth=20',input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
+          plot(np.flipud(smooth20[0,:,:]),ofile='smooth20',title='smooth,nsmooth=20')
 
-            smooth20 = cdo.smooth('nsmooth=20',input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
-            plot(np.flipud(smooth20[0,:,:]),ofile='smooth20',title='smooth,nsmooth=20')
+      def test_xarray_input(self):
+        cdo = Cdo(cdfMod='netcdf4')
+        try:
+          import xarray
+        except:
+          print("no xarray installation available!")
+          return
 
-        def test_xarray_input(self):
-          cdo = Cdo(cdfMod='netcdf4')
-          try:
-            import xarray
-          except:
-            print("no xarray installation available!")
-            return
-
-          dataSet = xarray.open_dataset(cdo.topo('global_0.1',options = '-f nc'))
+        dataSet = xarray.open_dataset(cdo.topo('global_0.1',options = '-f nc'))
 
+        if DEBUG:
           print(type(dataSet).__name__)
 
-          dataSet['topo'] = 1.0 + np.abs(dataSet['topo'])
+        dataSet['topo'] = 1.0 + np.abs(dataSet['topo'])
 
-          #check the changes withing xarray
-          self.assertEqual(1.0,np.min(dataSet['topo']))
+        #check the changes withing xarray
+        self.assertEqual(1.0,np.min(dataSet['topo']))
 
-          xarrayFile = 'test_xarray_topoAbs.nc'
-          dataSet.to_netcdf(xarrayFile)
+        xarrayFile = 'test_xarray_topoAbs.nc'
+        dataSet.to_netcdf(xarrayFile)
 
-          #check change via cdo
-          minByCdo = cdo.fldmin(input=xarrayFile,returnArray='topo').min()
-          self.assertEqual(1.0,minByCdo)
+        #check change via cdo
+        minByCdo = cdo.fldmin(input=xarrayFile,returnArray='topo').min()
+        self.assertEqual(1.0,minByCdo)
 
-          #do the same without explicit tempfile
-          self.assertEqual(1.0,cdo.fldmin(input=dataSet,returnArray='topo').min())
+        #do the same without explicit tempfile
+        self.assertEqual(1.0,cdo.fldmin(input=dataSet,returnArray='topo').min())
 
 
-        def test_xarray_output(self):
-          cdo = Cdo(cdfMod='netcdf4')
-          try:
-            import xarray
-          except:
-            print("no xarray installation available!")
-            return
+      def test_xarray_output(self):
+        cdo = Cdo(cdfMod='netcdf4')
+        try:
+          import xarray
+        except:
+          print("no xarray installation available!")
+          return
 
-          tArray = cdo.topo('global_10.0',options = '-f nc',returnXArray = 'topo')
+        tArray = cdo.topo('global_10.0',options = '-f nc',returnXArray = 'topo')
+        if DEBUG:
           print(tArray)
 
-        def test_xdataset_output(self):
-          cdo = Cdo(cdfMod='netcdf4')
-          try:
-            import xarray
-          except:
-            print("no xarray installation available!")
-            return
+      def test_xdataset_output(self):
+        cdo = Cdo(cdfMod='netcdf4')
+        try:
+          import xarray
+        except:
+          print("no xarray installation available!")
+          return
 
-          tDataset = cdo.topo('global_10.0',options = '-f nc',returnXDataset = True)
+        tDataset = cdo.topo('global_10.0',options = '-f nc',returnXDataset = True)
+        if DEBUG:
           print(tDataset)
 
 #===============================================================================
